﻿// Clumper - Actionscript 3 Version by Jim Bumgardner  9/6/2008

package 
{
	import flash.events.Event;
	import flash.display.Sprite;
	import flash.display.DisplayObject;
	import flash.display.GradientType;
	import flash.events.MouseEvent;
	import flash.geom.Matrix;

	public class Clumper_mc extends Sprite
	{
		static public const kNumberGuys = 50;     // Number of balls
	
		static private const kMinRadius = 12;       // Radius of balls
		static private const kMaxRadius = 36;      // Radius of balls
		static private const kPersonalSpace = 1.5; // Desired personal space 1.0 means balls are touching
							  // Used to extend collision radius beyond visible radius
		
		static private const kSpeed = 3.0;         // Controls how quickly balls move out of the way
		static private const kCenterWeight = 1/kNumberGuys; // Controls how strong the force towards the center is
		static private const kDamp = .8;           // Decreasing this makes the balls act like they are in thick oil
		static private const kRadiansToDegrees = 180/Math.PI;
		static private const kDegreesToRadians = Math.PI/180;
	
		static private var SW,SH;
		static private var guys = [];
		
		private var dragging:Boolean = false;
		private var vx,vy,lx,ly;
		public var radius;
	
		public static function InitBalls(itsPar:DisplayObject)
		{
			for (var j = 0; j < kNumberGuys; ++j) {
			  var mc = new Clumper_mc(itsPar);
			  itsPar.addChild(mc);
			}
		}
		
		public function Clumper_mc(itsPar:DisplayObject)
		{
			SW = itsPar.stage.stageWidth;
			SH = itsPar.stage.stageHeight;
			buttonMode = true; // use hand cursors
			guys.push(this);
			x = Math.random()*SW;
			y = Math.random()*SH;
			vx = vy = 0;
			radius = kMinRadius + Math.random()*(kMaxRadius - kMinRadius);
			drawSphere(getCandyColor(guys.length/(kNumberGuys-1)), radius);
			drawInvisibleCross(radius*kPersonalSpace);
			addEventListener(Event.ENTER_FRAME, this.clump_about);
			addEventListener(MouseEvent.MOUSE_DOWN, this.pressBall);
			addEventListener(MouseEvent.MOUSE_UP, this.releaseBall);
		}
	
		function pressBall(evt)
		{
		  dragging = true; 
		  lx = parent.root.mouseX;
		  ly = parent.root.mouseY; 
		  startDrag(true);
		}
	
		function releaseBall(evt)
		{
		  dragging = false;
		  stopDrag();
		}
	
		private function tintColor(a,v,mix)
		{
		  var r1 = a >> 16;
		  var g1 = (a >> 8) & 0xFF;
		  var b1 = a & 0xFF;
		  r1 = r1*(1-mix)+v*mix;
		  g1 = g1*(1-mix)+v*mix;
		  b1 = b1*(1-mix)+v*mix;
		  return (r1 << 16) | (g1 << 8) | b1;
		}
		
		private function getRainbowColor(r) // r is 0 - 1  - returns a bright color
		{
		  var a = r*Math.PI*2;
		  var r = 128+Math.cos(a)*127;
		  var b = 128+Math.cos(a+2*Math.PI/3)*127;
		  var g = 128+Math.cos(a+4*Math.PI/3)*127;
		  return (r << 16) | (g << 8) | b;
		}
		
		private function getCandyColor(r)
		{
		  var v = 128+Math.floor(Math.random()*127);
		  return tintColor(getRainbowColor(r),v,.5);
		}
	
		/* NOTE: this is no longer needed, since AS3 includes a drawCircle method
		private function drawCircle(x,y,radius)
		{
		  var r = radius;
		  var n= 8;
		  
		  var theta = 45*kDegreesToRadians;
		  var cr = radius/Math.cos(theta/2);
		  var angle = 0;
		  var cangle = angle-theta/2;
		  
		  graphics.moveTo(x+r, y);
		  for (var i=0;i < n;i++) 
		  {
			angle += theta;
			cangle += theta;
			var endX = r*Math.cos (angle);
			var endY = r*Math.sin (angle);
			var cX = cr*Math.cos (cangle);
			var cY = cr*Math.sin (cangle);
			graphics.curveTo(x+cX,y+cY, x+endX,y+endY);
		  }
		}*/
	
		private function drawInvisibleCross(r)
		{
		  graphics.lineStyle(1,0,0);
		  graphics.moveTo(-r,0);
		  graphics.lineTo(r,0);
		  graphics.moveTo(0,-r);
		  graphics.lineTo(0,r);
		}
	
		private function drawSphere(clr, r)
		{
		  var alphas = [100,100,100];
		  var ratios = [0x00,0x44,0xFF];
	      var colors = [0xFFFFFF,clr,tintColor(clr,0,.5)];

		  //      var matrix = { matrixType:"box", x:-r, y:-r*2, w:r*3, h:r*3, r: 0 }
		  var matrix = new Matrix();
		  matrix.createGradientBox(r*3, r*3, 0, -r, -r*2);
		
		  graphics.clear();
		  // graphics.beginFill(clr, 100);
	      graphics.beginGradientFill(GradientType.RADIAL,colors,alphas,ratios,matrix);
		  graphics.drawCircle(0,0,r);
		  graphics.endFill();
		}
		
		public function myHitTest(mc:Clumper_mc)
		{
			// old style test in comments: for rectangle intersection...
			// return mc.x-mc.width/2 < x+width/2 && mc.x+mc.width/2 >= x-width/2 &&
			//       mc.y-mc.height/2 < y+height/2 && mc.y+mc.height/2 >= y-width/2;
			var dx = mc.x - x;
			var dy = mc.y - y;
			var minDist = (radius+mc.radius);
			return (dx*dx+dy*dy) < minDist*minDist*2;
		}
		
		function clump_about(evt)
		{
		  if (dragging) {
			vx = parent.root.mouseX - lx;
			vy = parent.root.mouseY - ly;
			lx = parent.root.mouseX;
			ly = parent.root.mouseY;
		  }
		  else {
			vx *= kDamp;
			vy *= kDamp;
		  
			// Collision detection with balls (Avoidance)
			var weight = 0;
			var fx = 0;
			var fy = 0;
			var dx,dy,dist,scale;
			for (var i = guys.length-1; i >= 0; --i) 
			{
			  if (guys[i] != this && myHitTest(guys[i])) 
			  {
				dx = x - guys[i].x;
				dy = y - guys[i].y;
				dist = Math.sqrt(dx*dx+dy*dy);
				var maxDist = (this.radius+guys[i].radius)*kPersonalSpace;
				var diff = maxDist - dist;
				if (diff > 0) {
				  scale = diff/maxDist;
				  scale = scale*scale;
				  weight += scale;
				  scale = scale*kSpeed/dist;
				  fx += dx*scale;
				  fy += dy*scale;
				}
			  }
			}
		  
			// Attraction to center
			dx = SW/2 - x;
			dy = SH/2 - y;
			dist = Math.sqrt(dx*dx+dy*dy);
			weight += kCenterWeight;
			scale = kCenterWeight/dist;
			fx += dx*scale;
			fy += dy*scale;
		  
			vx += fx/weight;
			vy += fy/weight;
			
			this.x += vx;
			this.y += vy;
		  }
		  // Uncomment this to make object point in direction of travel
		  // this.rotation = Math.atan2(vx,vy)*kRadiansToDegrees;
		}

	}




}
