﻿// Clumping Simulation - Jim Bumgardner

kNumberGuys = 26;     // Number of balls
kMinRadius = 12;       // Radius of balls
kMaxRadius = 36;      // Radius of balls
kPersonalSpace = 1.5; // Desired personal space 1.0 means balls are touching
                      // Used to extend collision radius beyond visible radius

kSpeed = 3.0;         // Controls how quickly balls move out of the way
kCenterWeight = 1/kNumberGuys; // Controls how strong the force towards the center is
kDamp = .8;           // Decreasing this makes the balls act like they are in thick oil

SW = Stage.width;
SH = Stage.height;
kRadiansToDegrees = 180/Math.PI;
kDegreesToRadians = Math.PI/180;
guys = [];

function tintColor(a,v,mix)
{
	var r1 = a >> 16;
	var g1 = (a >> 8) & 0xFF;
	var b1 = a & 0xFF;
	r1 = r1*(1-mix)+v*mix;
	g1 = g1*(1-mix)+v*mix;
	b1 = b1*(1-mix)+v*mix;
	return (r1 << 16) | (g1 << 8) | b1;
}

getRainbowColor = function(r) // r is 0 - 1  - returns a bright color
{
	var a = r*Math.PI*2;
	var r = 128+Math.cos(a)*127;
	var b = 128+Math.cos(a+2*Math.PI/3)*127;
	var g = 128+Math.cos(a+4*Math.PI/3)*127;
	return (r << 16) | (g << 8) | b;
}

getCandyColor = function(r)
{
	var v = 128+random(127);
	return tintColor(getRainbowColor(r),v,.5);
}

MovieClip.prototype.drawCircle = function(x,y,radius)
{
	var r = radius;
	var n= 8;
	
	var theta = 45*kDegreesToRadians;
	var cr = radius/Math.cos(theta/2);
	var angle = bA*kDegreesToRadians;
	var cangle = angle-theta/2;
	
	this.moveTo(x+r, y);
	for (var i=0;i < n;i++) 
	{
		angle += theta;
		cangle += theta;
		var endX = r*Math.cos (angle);
		var endY = r*Math.sin (angle);
		var cX = cr*Math.cos (cangle);
		var cY = cr*Math.sin (cangle);
		this.curveTo(x+cX,y+cY, x+endX,y+endY);
	}
}

// This draws an invisible cross the size of the movieclip's
// 'personal space' - insuring that hittest returns true if there
// is an intersection of 'personal space'
//
MovieClip.prototype.drawInvisibleCross = function(r)
{
  this.lineStyle(1,0,0);
  this.moveTo(-r,0);
  this.lineTo(r,0);
  this.moveTo(0,-r);
  this.lineTo(0,r);
}

MovieClip.prototype.drawSphere = function(clr, r)
{
	var alphas = [100,100,100];
	var ratios = [0x00,0x44,0xFF];
  	var colors = [0xFFFFFF,clr,tintColor(clr,0,.5)];
	var matrix = { matrixType:"box", x:-r, y:-r*2, w:r*3, h:r*3, r: 0 }

	this.clear();
  	this.beginGradientFill("radial",colors,alphas,ratios,matrix);
	this.drawCircle(0,0,r);
	this.endFill();
}

function clump_about()
{
	if (this.dragging) {
		this.vx = _root._xmouse - this.lx;
		this.vy = _root._ymouse - this.ly;
		this.lx = _root._xmouse;
		this.ly = _root._ymouse;
	}
	else {
		this.vx *= kDamp;
		this.vy *= kDamp;
	
		// Collision detection with balls (Avoidance)
		var weight = 0;
		var fx = 0;
		var fy = 0;
		for (var i = guys.length-1; i >= 0; --i) 
		{
			if (guys[i] != this && this.hitTest(guys[i])) 
			{
				var dx = this._x - guys[i]._x;
				var dy = this._y - guys[i]._y;
				var dist = Math.sqrt(dx*dx+dy*dy);
				var maxDist = (this.radius+guys[i].radius)*kPersonalSpace;
				var diff = maxDist - dist;
				if (diff > 0) {
					var scale = diff/maxDist;
					scale = scale*scale;
					weight += scale;
					scale = scale*kSpeed/dist;
					fx += dx*scale;
					fy += dy*scale;
				}
			}
		}
	
		// Attraction to center
		var dx = SW/2 - this._x;
		var dy = SH/2 - this._y;
		var dist = Math.sqrt(dx*dx+dy*dy);
		weight += kCenterWeight;
		var scale = kCenterWeight/dist;
		fx += dx*scale;
		fy += dy*scale;
	
		this.vx += fx/weight;
		this.vy += fy/weight;
		
		this._x += this.vx;
		this._y += this.vy;
	}

	// Uncomment this to make object point in direction of travel
	// this._rotation = Math.atan2(this.vx,this.vy)*kRadiansToDegrees;
}

// Setup critterrs in random positions
for (var i = 0; i < kNumberGuys; ++i) 
{
	var mc = this.createEmptyMovieClip("critter_mc"+i, i+1);
	guys.push(mc);
	mc.idx = i;
	mc._x = random(SW);  // critter position (random)
	mc._y = random(SH);
	mc.vx = 0;
	mc.vy = 0;
	mc.radius = kMinRadius + Math.random()*(kMaxRadius - kMinRadius);
	mc.drawSphere(getCandyColor(i/(kNumberGuys-1)), mc.radius);
	mc.drawInvisibleCross(mc.radius*kPersonalSpace);
	mc.onEnterFrame = clump_about; 
	mc.onPress = function() { this.dragging = true; this.lx = _root._xmouse; this.ly = _root._ymouse; this.startDrag(true); };
	mc.onRelease = mc.onReleaseOutside = function() { this.dragging = false; this.stopDrag(true); };
}

/****
// Performance testing - Display framerate every 10 seconds

count = 0;
st = getTimer();
_root.onEnterFrame = function()
{
	++count;
	if (getTimer() - st >= 10*1000) {
		trace("fps = " + count/10);
		st = getTimer();
		count = 0;
	}
}

****/